# -*- coding: utf-8 -*-

"""Test cases for utility functions"""
from __future__ import absolute_import
from builtins import str

import pytest
import os
import errno
import os.path as op
import stat
import flotsam.file_util as sut
from flotsam.pytest_util import unique_filename

_ = unique_filename


@pytest.fixture()
def existent_empty_file(tmpdir):
    filename = op.join(str(tmpdir), 'file_1.txt')
    with open(filename, 'w'):
        pass
    return filename


def test_ensure_dir_new(unique_filename, tmpdir):
    path = str(tmpdir) + unique_filename
    assert not op.exists(path)
    sut.ensure_dir(path)
    assert op.exists(path)
    assert os.stat(path).st_mode & stat.S_IRWXU == stat.S_IRWXU


@pytest.mark.parametrize('mode',
                         [0,
                          stat.S_IRUSR,
                          stat.S_IWUSR,
                          stat.S_IXUSR])
def test_ensure_dir_missing_perm(mode, unique_filename, tmpdir):
    path = str(tmpdir) + unique_filename
    assert not op.exists(path)
    os.mkdir(path)
    assert op.exists(path)
    os.chmod(path, mode)
    try:
        sut.ensure_dir(path)
    except IOError as e:
        assert e.errno == errno.EPERM
    else:
        assert False, 'Should have raised IOError'
    assert op.exists(path)


def test_ensure_dir_existing(unique_filename, tmpdir):
    path = str(tmpdir) + unique_filename
    assert not op.exists(path)
    os.mkdir(path)
    assert op.exists(path)
    sut.ensure_dir(path)
    assert op.exists(path)


def test_ensure_del_missing_file(tmpdir):
    filename = op.join(str(tmpdir), '__NONEXISTANT')
    sut.ensure_deleted(filename)
    assert not op.exists(filename)


def test_ensure_del_missing_dir(tmpdir):
    dirname = str(tmpdir)
    sut.ensure_deleted(dirname)
    assert not op.exists(dirname)


def test_ensure_del_ex_file(existent_empty_file):
    sut.ensure_deleted(existent_empty_file)


def test_ensure_del_ex_dir(tmpdir):
    sut.ensure_deleted(str(tmpdir))



# -*- coding: utf-8 -*-

"""Utilities for collections"""


def pretty_dict(dic):
    """Printable string representation of a dictionary

       Items are listed in alphabetical order of their keys,
       to ensure a deterministic representation
       (e.g. for unit tests)

       Examples:

       >>> pretty_dict({1: 2, 3: 4})
       '{1: 2, 3: 4}'

       >>> pretty_dict({'one': 'two', 'three': 'four'})
       '{one: two, three: four}'

       Recursive dictionaries are supported, too:

       >>> pretty_dict({1: 2, 3: {4: 5, 6: 7}})
       '{1: 2, 3: {4: 5, 6: 7}}'
    """
    items = []
    keys = sorted(dic.keys())
    for key in keys:
        val = dic[key]
        if isinstance(val, dict):
            val = pretty_dict(val)
        item_str = '{0}: {1}'.format(key, val)
        items.append(item_str)
    return '{' + ', '.join(items) + '}'



# -*- coding: utf-8 -*-

"""Utilities related to py.test"""

import uuid
import pytest


@pytest.fixture
def unique_filename():
    """Creates a UUID-based unique filename"""
    return str(uuid.uuid1())



import sys
from setuptools import setup
#from distutils.core import setup
from setuptools.command.test import test as TestCommand

readme = open('README.rst').read()
history = open('HISTORY.rst').read()


def parse_requirements():
    with open('requirements.txt') as req:
        return req.readlines()


class Tox(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import tox
        errcode = tox.cmdline(self.test_args)
        sys.exit(errcode)


setup(name='flotsam',
      version='0.1.2',
      description='Miscellaneous utilities',
      long_description=readme + '\n\n' + history,
      author='Bernhard (Bernd) Biskup',
      author_email='bbiskup@gmx.de',
      url='https://github.com/bbiskup',
      package_dir={'flotsam': 'flotsam'},
      packages=['flotsam'],
      zip_safe=False,
      include_package_data=True,
      install_requires=parse_requirements(),
      classifiers=[
          'Development Status :: 2 - Pre-Alpha',
          'Environment :: Console',
          'Intended Audience :: Developers',
          # Topic :: Software Development :: Libraries :: Python Modules',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.5',
      ],
      license='The MIT License (MIT)',
      keywords='flotsam utilities',
      tests_require=['tox'],
      cmdclass={'test': Tox}
      )



import flotsam.collection_util as sut

import pytest


@pytest.fixture
def int_dict():
    return {3: 4, 1: 2}


@pytest.fixture
def str_dict():
    return {'three': 'four', 'one': 'two'}


class Dummy:

    """Dummy class to test dictionary of instances
    """

    def __init__(self, val):
        self._val = val

    def __repr__(self):
        return 'Dummy_{}'.format(self._val)


@pytest.fixture
def instance_dict():
    return {3: Dummy(1), 1: Dummy(2)}


@pytest.fixture
def dict_dict():
    return {3: {7: 8, 5: 6}, 1: {11: 12, 9: 10}}


def test_empty():
    assert sut.pretty_dict({}) == '{}'


def test_ints(int_dict):
    expected = "{1: 2, 3: 4}"
    assert sut.pretty_dict(int_dict) == expected


def test_strings(str_dict):
    expected = "{one: two, three: four}"
    assert sut.pretty_dict(str_dict) == expected


def test_instances(instance_dict):
    expected = "{1: Dummy_2, 3: Dummy_1}"
    assert sut.pretty_dict(instance_dict) == expected


def test_dict_dict(dict_dict):
    expected = "{1: {9: 10, 11: 12}, 3: {5: 6, 7: 8}}"
    assert sut.pretty_dict(dict_dict) == expected



# -*- coding: utf-8 -*-

"""Miscellaneous utilities related to files an directories"""
from __future__ import print_function

import os
import os.path as op
import stat
import errno
import shutil


def ensure_dir(path):
    """Make sure the specified directory exists
       and is writable
    """
    try:
        os.mkdir(path)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise

    if os.stat(path).st_mode & stat.S_IRWXU != stat.S_IRWXU:
        raise IOError(errno.EPERM,
                      'Insufficient directory permissions',
                      path)


def _ensure_file_deleted(path):
    """Try to delete a file; don't raise an exception if it does not exist"""
    try:
        os.remove(path)
    except OSError as e:
        if e.errno != errno.ENOENT:
            raise


def _ensure_dir_deleted(path):
    """Try to delete a directory;
       don't raise an exception if it does not exist
       """
    try:
        shutil.rmtree(path)
    except OSError as e:
        if e.errno != errno.ENOENT:
            raise


def ensure_deleted(path):
    """Ensure the given file or directory is deleted;
       don't raise an exception if it does not exist.
       """
    if not op.exists(path):
        return
    if op.isfile(path):
        _ensure_file_deleted(path)
    elif op.isdir(path):
        _ensure_dir_deleted(path)
    else:
        raise ValueError('Invalid file type: {}'.format(path))
